#include <stdio.h>
#include <stdlib.h>

#ifndef INT_MAX
#define INT_MAX ( (signed int)(~(unsigned int)0 >> 1))
#endif
#ifndef INT_MIN
#define INT_MIN (~(signed int)(~(unsigned int)0 >> 1))
#endif

#include "CGraph.hpp"

using namespace std;

CGraph::CGraph( int _vertexcnt, int _edgecnt )
 {
	vertexcnt = _vertexcnt;
	edgecnt = _edgecnt;
	nodes = (TVertex *)new TVertex[vertexcnt];
	edges = (TEdge *)new TEdge[edgecnt];
	for( int i = 0; i < vertexcnt; i++ )
		nodes[i].routecnt = 0;
 }

CGraph::~CGraph()
 {
	for( int i = 0; i < vertexcnt; i++ )
		if( nodes[i].routecnt > 0 )
			delete [] nodes[i].routes;
	if( vertexcnt > 0 ) delete [] nodes;
	if( edgecnt   > 0 ) delete [] edges;
 }

void CGraph::makeRoutes()
 {
	for( int i = 0; i < vertexcnt; i++ )
	 {
		int routeidx = 0;
		for( int j = 0; j < edgecnt; j++ )
			if( edges[j].vex1 == i || edges[j].vex2 == i ) routeidx++;
		if( routeidx == 0 ) continue;
		nodes[i].routecnt = routeidx;
		nodes[i].routes = new TRoute[nodes[i].routecnt];
		routeidx = 0;
		for( int j = 0; j < edgecnt; j++ )
		 {
			if( edges[j].vex1 == i )
			 {
				nodes[i].routes[routeidx].edgeId = j;
				nodes[i].routes[routeidx].nextVertex = edges[j].vex2;
				routeidx++;
			 }
			else if( edges[j].vex2 == i )
			 {
				nodes[i].routes[routeidx].edgeId = j;
				nodes[i].routes[routeidx].nextVertex = edges[j].vex1;
				routeidx++;
			 }
		 }
	 }
	return;
 }

int CGraph::skeleton()
 {
	int i, segments = 0;

	clearVTags();
	clearETags();
	for( i = 0; i < edgecnt; i++ ) edges[i].bone = false;
	while( ( i = getCentroid( true ) ) != -1 )
	 {
		nodes[i].tag = 1;
		skeleton_R( i );
		segments++;
	 }
	return segments;
 }

void CGraph::skeleton_R( int beg )
 {
	for( int i = 0; i < nodes[beg].routecnt; i++ )
		if( nodes[nodes[beg].routes[i].nextVertex].tag == 0 )
		 {
			nodes[nodes[beg].routes[i].nextVertex].tag = 1;
			edges[nodes[beg].routes[i].edgeId].bone = true;
		 }
	for( int i = 0; i < nodes[beg].routecnt; i++ )
		if( edges[nodes[beg].routes[i].edgeId].tag == 0 )
		 {
			edges[nodes[beg].routes[i].edgeId].tag = 1;
			skeleton_R( nodes[beg].routes[i].nextVertex );
		 }
	return;
 }

int CGraph::bones()
 {
	int tailbones = 0;

	for(int i = 0; i < vertexcnt; i++ )
	 {
		nodes[i].bones = 0;
		for( int j = 0; j < nodes[i].routecnt; j++ )
			if( edges[nodes[i].routes[j].edgeId].bone ) nodes[i].bones++;
		if( nodes[i].bones == 1 ) tailbones++;
	 }
	return tailbones;
 }
 
void CGraph::clearVTags()
 {
	for( int i = 0; i < vertexcnt; i++ )
		nodes[i].tag = 0;
	return;
 }

void CGraph::clearETags()
 {
	for( int i = 0; i < edgecnt; i++ )
		edges[i].tag = 0;
	return;
 }

bool CGraph::isTail    ( int vertexId ) const {return ( nodes[vertexId].routecnt == 1 );}
bool CGraph::isTailBone( int vertexId ) const {return ( nodes[vertexId].bones    == 1 );}

int CGraph::getCentroid( bool untaggedOnly ) const
 {
	int cntr = -1;
	int i;
	
	for( i = 0; i < vertexcnt; i++ )
		if( !untaggedOnly || nodes[i].tag == 0 )
			{cntr = i; break;}
	for( ++i; i < vertexcnt; i++ )
		if( !untaggedOnly || nodes[i].tag == 0 )
			if( nodes[i].routecnt > nodes[cntr].routecnt ) cntr = i;
	return cntr;
 }

void CGraph::getLoop( int edgeAround )
 {
	clearETags();
	clearVTags();
	nodes[edges[edgeAround].vex2].tag = 1;
	if( getLoop_R( edges[edgeAround].vex2, edges[edgeAround].vex1 ) )
		edges[edgeAround].tag = 1;
	return;
 }

bool CGraph::getLoop_R( int beg, int end )
 {
	if( beg == end ) return true;
	else
	 {
		for( int i = 0; i < nodes[beg].routecnt; i++ )
			if( edges[nodes[beg].routes[i].edgeId].bone && nodes[nodes[beg].routes[i].nextVertex].tag == 0 )
			 {
				nodes[nodes[beg].routes[i].nextVertex].tag = 1;
				if( getLoop_R( nodes[beg].routes[i].nextVertex, end ) )
				 {
					edges[nodes[beg].routes[i].edgeId].tag = edges[nodes[beg].routes[i].edgeId].vex1 == beg ? 1 : -1;
					return true;
				 }
			 }
	 }
	return false;
 }

void CGraph::getCut( int boneThru )
 {
	if( isTailBone( edges[boneThru].vex1 ) )
	 {
	  	clearETags();
		for( int i = 0; i < nodes[edges[boneThru].vex1].routecnt; i++ )
			edges[nodes[edges[boneThru].vex1].routes[i].edgeId].tag 
				= edges[nodes[edges[boneThru].vex1].routes[i].edgeId].vex1 == edges[boneThru].vex1 ? 1 : -1;
		return;
	 }
	if( isTailBone( edges[boneThru].vex2 ) )
	 {
		clearETags();
		for( int i = 0; i < nodes[edges[boneThru].vex2].routecnt; i++ )
			edges[nodes[edges[boneThru].vex2].routes[i].edgeId].tag
				= edges[nodes[edges[boneThru].vex2].routes[i].edgeId].vex2 == edges[boneThru].vex2 ? 1 : -1;
		return;
	 }
	splitTree( boneThru, 1, 2 );
	for( int i = 0; i < edgecnt; i++ )
	 {
		if( nodes[edges[i].vex1].tag + nodes[edges[i].vex2].tag == 3 )
			edges[i].tag = nodes[edges[i].vex1].tag == 1 ? 1 : -1;
	 }
	return;
 }

void CGraph::splitTree(int border, int mark1, int mark2 )
 {
	clearVTags();
	clearETags();
	nodes[edges[border].vex2].tag = mark1;
	splitTree_R(edges[border].vex1, mark1);
	nodes[edges[border].vex1].tag = mark2;
	splitTree_R(edges[border].vex2, mark2);
	nodes[edges[border].vex1].tag = mark1;
	return;
 }

void CGraph::splitTree_R( int beg, int mark )
 {
	for( int i = 0; i < nodes[beg].routecnt; i++ )
	 {
		if( edges[nodes[beg].routes[i].edgeId].bone )
			if( nodes[nodes[beg].routes[i].nextVertex].tag != mark )
			 {
				nodes[nodes[beg].routes[i].nextVertex].tag = mark;
				splitTree_R( nodes[beg].routes[i].nextVertex, mark );
			 }
	 }
	return;
 }

static CGraph *cmpweight_hint;   // cmpweight musi byt staticka kvuli volani qsort a zaroven potrebuje vedet, nad kterou instanci grafu pracovat
static int cmpweight( const void *index1, const void *index2 )
 {
	if( cmpweight_hint->edges[*(int *)index1].weight > cmpweight_hint->edges[*(int *)index2].weight ) return  1;
	if( cmpweight_hint->edges[*(int *)index1].weight < cmpweight_hint->edges[*(int *)index2].weight ) return -1;
	return 0;
 }

int CGraph::BoruvkaKruskal( bool detailed )   // navratovou hodnotou je pocet komponent grafu
 {
	int i, j, segments = 0;
	int *sortbuf;   // pole indexu hran, ktere bude obsahovat serazenou posloupnost podle vahy
 
	sortbuf = new int[edgecnt];
	for( i = 0; i < edgecnt; ++i ) sortbuf[i] = i;   // inicializace identickou permutaci
	cmpweight_hint = this;
	if( detailed ) fprintf( stdout, "Boruvka-Kruskal: radim pole hran podle jejich vah\n" );
	qsort( (void *)sortbuf, edgecnt, sizeof(*sortbuf), cmpweight );   // serazeni indexu podle vah hran (vzestupne)
	
	// tagy uzlu budou nazvy mnozin jejich prislusnosti, pocatecni nastaveni je kazdy uzel ve vlastni mnozine
	for( i = 0; i < vertexcnt; ++i ) nodes[i].tag = i;
	for( i = 0; i < edgecnt; ++i ) edges[i].bone = false;

	for( i = 0; i < edgecnt; ++i )
	 {
		// hrana se bude pridavat pokud jeji uzly nepatri do stejne mnoziny
		if( nodes[edges[sortbuf[i]].vex1].tag != nodes[edges[sortbuf[i]].vex2].tag )
		 {
			if( detailed )
			 {
				printf( "Boruvka-Kruskal: sjednocuji podstromy kostry pres hranu %d s vahou %d, \n", 
				        sortbuf[i], edges[sortbuf[i]].weight ); 
				printf( "Boruvka-Kruskal: ...uzly podstromu %d (uzel %d) vkladam do mnoziny %d (uzel %d)\n", 
				        nodes[edges[sortbuf[i]].vex2].tag, edges[sortbuf[i]].vex2, 
				        nodes[edges[sortbuf[i]].vex1].tag, edges[sortbuf[i]].vex1 ); 
			 }
			edges[sortbuf[i]].bone = true;
			int uniontag = nodes[edges[sortbuf[i]].vex2].tag;
			for( j = 0; j < vertexcnt; ++j )
			 {
				// sjednoceni mnozin - mnozine s druhym vrcholem pridavane hrany se nastavi jmeno mnoziny s prvnim vrcholem
				if( nodes[j].tag == uniontag ) nodes[j].tag = nodes[edges[sortbuf[i]].vex1].tag;
			 }
		 }
	 }
	 delete [] sortbuf;
	// urceni poctu komponent (segmentu) - je to pocet skupin s jinym nazvem mnoziny, do ktere prislusi
	for( i = 0; i < vertexcnt; ++i )
	 {
		if( nodes[i].tag != -1 )   // uzly s nenulovym tagem patri do kostry komponenty
		 {
			++segments;
			int cleartag = nodes[i].tag;
			for( j = 0; j < vertexcnt; ++j ) if( nodes[j].tag == cleartag ) nodes[j].tag = -1;   // odoznaceni cele komponenty
		 }
	 }
	if( detailed ) fprintf( stdout, "Boruvka-Kruskal: pocet komponent grafu a nalezene kostry = %d\n", segments );
	return segments;
 }

int CGraph::JarnikPrim( bool detailed )   // navratovou hodnotou je pwocet komponent grafu
 {
 	int i, j, segments = 0;
	int *pred;
	int start = 0;   // uzel, ze ktereho bude algoritmus vychazet (pro dalsi komponentu to bude prvni, ktery zustane nedosazitelny)

	pred = new int[vertexcnt];   // indexy hran k predchudcum uzlu v ramci jejich priblizovani ke vznikajici kostre
	for( i = 0; i < vertexcnt; ++i ) pred[i] = -1;
	// v tagu uzlu je jeho vzdalenost k podstromu vznikajici kostry, zpocatku nekonecno krome prvniho
	for( i = 0; i < vertexcnt; ++i ) nodes[i].tag = INT_MAX;
	for( i = 0; i < edgecnt; ++i ) edges[i].bone = false;
 
 nextSegment:
	nodes[start].tag = 0;
	while( 17 )
	 {
		j = -1;   // j bude index uzlu, ktery je nejblize podstromu vznikajici kostry, zpocatku neplatny index, protoze takovy uzel nemusi existovat
		for( i = 0; i < vertexcnt; ++i )
		 {
			if( nodes[i].tag == INT_MIN ) continue;   // minimalni strojova hodnota znaci, ze uzel se jiz nenachazi ve fronte ke zpracovani
			if( nodes[i].tag == INT_MAX ) continue;   // maximalni strojova hodnota znaci, ze uzel se je mimo viditelnost (daleko nebo v jine komponente)
			if( j < 0 ) j = i;   // prvni test projde
			if( nodes[i].tag < nodes[j].tag ) j = i;
		 }
		if( j < 0 ) break;   // platny index je >= 0, jinak uz v dane komponente nezbyvaji zadne dalsi uzly ke zpracovani
		if( detailed ) fprintf( stdout, "Jarnik-Prim: dalsi uzel nejblize podstromu vznikajici kostry je %d\n", j );
		for( i = 0; i < nodes[j].routecnt; ++i )
		 {
			// aktualizace ohodnoceni nasledniku zpracovavaneho uzlu (pokud existuje kratsi cesta z podstromu skrz hranu od zpracovavaneho uzlu)
			if( nodes[nodes[j].routes[i].nextVertex].tag == INT_MIN ) continue;
			if( nodes[nodes[j].routes[i].nextVertex].tag > edges[nodes[j].routes[i].edgeId].weight )
			 {
				nodes[nodes[j].routes[i].nextVertex].tag = edges[nodes[j].routes[i].edgeId].weight;
				pred[nodes[j].routes[i].nextVertex] = nodes[j].routes[i].edgeId;
				if( detailed )
					fprintf( stdout, "Jarnik-Prim: ...pak uzel %d je od podstromu vzdalen %d a to pres hranu %d\n", 
					        nodes[j].routes[i].nextVertex, edges[nodes[j].routes[i].edgeId].weight, nodes[j].routes[i].edgeId );
			 }
		 }
		nodes[j].tag = INT_MIN;   // uzel se oznaci jako zpracovany (vyradi se z "fronty")
	 }
	
	// nyni jsme zpracovali jednu komponentu (tu s uzlem 0), ostatni nedosazitelne maji v tagu nekonecnou vzdalenost (INT_MAX) => provedeme algoritmus pro dalsi
	++segments;
	for( i = 0; i < vertexcnt; ++i )
		if( nodes[i].tag == INT_MAX )
		 {
			start = i;
			goto nextSegment;
		 }
	// nastaveni kostrovych priznaku hran
	for( i = 0; i < vertexcnt; ++i )
	 {
		if( pred[i] != -1 )
			edges[pred[i]].bone = true;   // vlastni vlozeni hrany (vedouci do nejblizsiho uzlu) do nalezene kostry
	 }
	delete [] pred;
	if( detailed ) fprintf( stdout, "Jarnik-Prim: pocet komponent grafu a nalezene kostry = %d\n", segments );
	return 0;
 }
