#ifndef __CGRAPH_HPP__
#define __CGRAPH_HPP__

#ifndef NULL
#define NULL 0x00
#endif


class CGraph
 {
  public:
	struct TEdge
	 {
		int vex1;
		int vex2;
		bool bone;
		int tag;
		int weight;
	 };   // default orientation of edge is from vex1 to vex2
	struct TRoute
	 {
		int edgeId;
		int nextVertex;
	 };
	struct TVertex
	 {
		struct TRoute *routes;
		int routecnt;
		int bones;
		int tag;
	 };

	int vertexcnt;
	int edgecnt;
	struct TVertex *nodes;
	struct TEdge *edges;

	CGraph( int _vertexcnt, int _edgecnt );
	~CGraph();
	
	void makeRoutes();
	int skeleton();
	int bones();
	void clearVTags();
	void clearETags();
	bool isTail    ( int vertexId ) const;
	bool isTailBone( int vertexId ) const;
	int	getCentroid( bool untaggedOnly ) const;
	void getLoop( int edgeAround );   // mark Loop around non-bone edge, edges are marked +1 if they loop in default orientation (and vice versa), edgeAround is treated positive
	void getCut( int boneThru );   // mark Cut thru bone edge, edges are marked +1/-1 if they lead as/against boneThru edge, which is treated positive
	void splitTree( int border, int mark1, int mark2 );
	int BoruvkaKruskal( bool detailed );   // detailed will make printouts of some relevant actions
	int JarnikPrim    ( bool detailed );   // detailed will make printouts of some relevant actions
	
  protected: 
	void skeleton_R( int );
	bool getLoop_R( int, int );
	void splitTree_R( int, int );
 };


#endif
