#include <stdio.h>
#include <stdlib.h>

#include "algebra.h"
#include "CGraph.hpp"

using namespace std;

int main( int argc, char *argv[] )
 {
	char buf[256];
	int uzly;
	int hrany;
	int algoritmus;
	int argument;
	FILE *soubor = stdin;
	
	if( argc > 1 )
	 {
		soubor = fopen( argv[1], "r" );
		if( soubor == NULL )
		 {
			fprintf( stdout, "CHYBA: nelze otevrit soubor \"%s\"\n", argv[1] );
			exit(2);
		 } else {
			fprintf( stdout, "nacitam strukturu grafu ze souboru  %s... ", argv[1] );
		 }
	 }

	do {
		if( soubor == stdin ) fputs( "pocet hran: ", stdout );
		fgets( buf, sizeof( buf ), soubor );
		sscanf( buf, "%d\n", &hrany );
	 } while( hrany < 0 );
	do {
		if( soubor == stdin ) fputs( "pocet uzlu : ", stdout );
		fgets( buf, sizeof( buf ), soubor );
		sscanf( buf, "%d\n", &uzly );
	 } while( uzly <= 0 );
		
	CGraph graf( uzly, hrany );
	
	for( int i = 0; i < hrany; i++ )
	 {
		do {
			if( soubor == stdin ) fprintf( stdout, "hrana %d od uzlu: ", i );
			fgets( buf, sizeof( buf ), soubor );
			sscanf( buf, "%d\n", &(graf.edges[i].vex1) );
		 } while( !betweend( 0, graf.edges[i].vex1, uzly-1 ) );
		do {
			if( soubor == stdin ) fprintf( stdout, "hrana %d do uzlu: ", i );
			fgets( buf, sizeof( buf ), soubor );
			sscanf( buf, "%d\n", &(graf.edges[i].vex2) );
		 } while( !betweend( 0, graf.edges[i].vex2, uzly-1 ) );
		 {
			if( soubor == stdin ) fprintf( stdout, "hrana %d ma vahu: ", i );
			fgets( buf, sizeof( buf ), soubor );
			sscanf( buf, "%d\n", &(graf.edges[i].weight) );
		 }
	 }
	if( soubor != stdin )
	 {
		if( feof( soubor ) )   // na konci validniho souboru je prazdny radek, ktery je absorbovan v pripade spatneho zadani cisel (a tudiz opakovani dotazu)
		 {
			fputs( "[CHYBA] (nekonzistentni data)\n", stdout );
			exit(1);
		 } else fputs ( "[OK]\n", stdout );
	 }
	fputc( '\n', stdout );
	fputs( "vyber algoritmu:\n"
	       " 1 - rekurzivni kostra od uzlu s nejvyssim stupnem\n"
	       " 2 - Boruvka-Kruskal s urcenim poctu komponent\n"
	       " 3 - Jarnik-Prim pro nesouvisly graf\n"
	       " 4 - nezavisla smycka\n"
	       " 5 - nezavisly rez\n", stdout );
	do {
		fgets( buf, sizeof( buf ), stdin );
		sscanf( buf, "%d\n", &algoritmus );
	 } while( !betweend( 1, algoritmus, 5 ) );
	
	graf.makeRoutes();
	switch( algoritmus )
	 {
	  case 1:
		graf.skeleton();
		graf.bones();   // informace o kostre by pro nektere algoritmy mely byt prtomny i u uzlu (pocet incidujicich hran kostry)
		break;
	  case 2:
		do {
			fputs( "podrobny vypis (a/n)? ", stdout );
			fgets( buf, sizeof( buf ), stdin );
		 } while( buf[0] != 'a' && buf[0] != 'n' );
		if ( buf[0] == 'a' ) graf.BoruvkaKruskal( true ); else graf.BoruvkaKruskal( false );
		graf.bones();
		break;
	  case 3:
		do {
			fputs( "podrobny vypis (a/n)? ", stdout );
			fgets( buf, sizeof( buf ), stdin );
		 } while( buf[0] != 'a' && buf[0] != 'n' );
		if ( buf[0] == 'a' ) graf.JarnikPrim( true ); else graf.JarnikPrim( false );
		graf.bones();
		break;
	  case 4:
	  case 5:
		do {
			fputs( "cislo algoritmu pro oznaceni kostry (1-3): ", stdout );
			fgets( buf, sizeof( buf ), stdin );
			sscanf( buf, "%d\n", &argument );
		 } while( !betweend( 1, argument, 3 ) );
		if     ( argument == 1 ) graf.skeleton();
		else if( argument == 2 ) graf.BoruvkaKruskal( false );
		else if( argument == 3 ) graf.JarnikPrim( false );
		graf.bones();
		for( int i = 0; i < graf.edgecnt; ++i ) fprintf( stdout, "hrana %2d: v kostre=%d\n", i, graf.edges[i].bone );
		if( algoritmus == 4 )
		 {
			do {
				fputs( "nekostrova hrana okolo niz se najde nezavisla smycka: ", stdout );
				fgets( buf, sizeof( buf ), stdin );
				sscanf( buf, "%d\n", &argument );
			 } while ( !betweend( 0, argument, graf.edgecnt ) || graf.edges[hrany].bone );
			graf.getLoop( argument );
		 } else {   // algoritmus == 5
			do {
				fputs( "kostrova hrana skrz kterou se najde nezavisly rez: ", stdout );
				fgets( buf, sizeof( buf ), stdin );
				sscanf( buf, "%d\n", &argument );
			 } while ( !betweend( 0, argument, graf.edgecnt ) || !graf.edges[argument].bone );
			graf.getCut( argument );
		 }
		break;
	 }

	fputc( '\n', stdout );
	fputs( ">>VYSTUP<<\n", stdout );
	switch( algoritmus )
	 {
	  case 1:
	  case 2:
	  case 3:
		for( int i = 0; i < graf.edgecnt; ++i )
		 {
			fprintf( stdout, "hrana %2d: v kostre=%2d, vaha=%2d\n", i, graf.edges[i].bone, graf.edges[i].weight );
		 }
		for( int i = 0; i < graf.vertexcnt; ++i )
		 {
			fprintf( stdout, "uzel  %2d: kosti z=%2d, smeru=%2d\n", i, graf.nodes[i].bones, graf.nodes[i].routecnt );
		 }		
		break;
	 case 4:
	 case 5:
		for( int i = 0; i < graf.edgecnt; ++i )	fprintf( stdout, "hrana %2d: orientace=%2d\n", i, graf.edges[i].tag );
		break;
	 }

	 return 0;
 }
